<?php
// $Id$

/**
 * @file
 * OM Maximenu Admin Utilities
 *
 * @author: Daniel Honrade http://drupal.org/user/351112
 *
 */
define('OM_MAXIMENU_PATH', drupal_get_path('module', 'om_maximenu')); 


/**
 * Implementation of hook_init().
 *
 */
function om_maximenu_init() {
  global $_om_maximenu_variable;
  global $_om_maximenu_block_classes;  
  
  // add om tools custom block classes    
  if (module_exists('om_tools')) {

    $om_tools_values = variable_get('om_tools', '');
    $om_blocks = isset($om_tools_values['block']) ? $om_tools_values['block']: array();
    
    if (!empty($om_blocks) && ($om_blocks['block_classes_switch'] == 1)) {
      $_om_maximenu_block_classes = array();
      foreach ($om_blocks as $block_key => $block_class) {
        $block  = explode('___', $block_key);
        if (isset($block[1])) { // filters switch/reset
          $module = $block[1];
          $delta  = $block[2];
          $_om_maximenu_block_classes[$module][$delta] = $block_class;
        }
      }
    }
  }  
  
  $_om_maximenu_variable  = variable_get('om_maximenu', array());  
}

  
/**
 * Loads styles and js
 *
 */
function om_maximenu_asset_loader($menu_content = array()) {
  // options
  $action        = isset($menu_content['action']) ? $menu_content['action']: 'hover';  
  $displace      = isset($menu_content['displace']) ? $menu_content['displace']: 0;  
  $skin          = isset($menu_content['skin']) ? $menu_content['skin']: 'bubble';
  $animated_bg   = isset($menu_content['animated_bg']) ? 1: 0; 
  $animated_link = isset($menu_content['animated_link']) ? 1: 0; 
  $jiggle        = isset($menu_content['jiggle']) ? 1: 0; 
  $scroll        = isset($menu_content['scroll']) ? 1: 0;                       
  $tabbed        = isset($menu_content['tabbed']) ? $menu_content['tabbed']: '';                                                     
  $dock          = isset($menu_content['float_options']['dock']) ? 1: 0;
  $maximenu_name = om_string_name($menu_content['title']); 

  // add action per OM Maximenu
  om_maximenu_action($action, $maximenu_name, $displace); 
              
  // add css per OM Maximenu     
  drupal_add_css(OM_MAXIMENU_PATH . '/skin/' . $skin . '/' . $skin . '.css');
      
  // animated bg effect
  if ($animated_bg) om_maximenu_bg_slider($maximenu_name); 

  // animated link effect
  if ($animated_link) om_maximenu_link_slider($maximenu_name); 

  // animated link jiggle
  if ($jiggle) om_maximenu_jiggle($maximenu_name);

  // scrolling float
  if ($scroll) om_maximenu_scroll($maximenu_name);

  // tabbed content
  if ($tabbed) om_maximenu_tabbed($maximenu_name, $tabbed);
        
  // jquery fisheye dock effect        
  if ($dock) om_maximenu_dock($maximenu_name);
}


/**
 * Load all OM Maximenu blocks
 * 
 * This will process all created OM Maximenu blocks
 *
 */
function om_maximenu_block_load($maximenu_delta = '') {
  global $_om_maximenu_variable;
  $maximenu = $_om_maximenu_variable;
   
  $maximenu_delta = explode('-', $maximenu_delta);
  // ex. $maximenu_delta = 'om-maximenu-1';
  $delta = $maximenu_delta[2];
  
  $out = array();
  // om delta always starts with 1
  if (isset($maximenu[$delta]) && ($delta != 0)) {
    if (!empty($maximenu) && ($maximenu[$delta]['output'] == 'block')) {
      $visibility_values = isset($maximenu[$delta]['menu_visibility']) ? $maximenu[$delta]['menu_visibility']: array('visibility' => 0, 'pages' => '');
      $visibility = om_maximenu_menu_visibility($visibility_values);
      if ($visibility) { 
        // effects won't load if disabled blocks 
        if (!empty($maximenu[$delta]['block_options']['region'])) om_maximenu_asset_loader($maximenu[$delta]);
        $out['title']   = check_plain($maximenu[$delta]['title']);
        $out['content'] = om_maximenu_links_render($maximenu[$delta], $delta);
      }
    }
  }
  return $out;  
}


/**
 * Load all OM Maximenu blocks
 * 
 * This will process the main menu and floating blocks
 *
 */
function om_maximenu_load() {
  global $_om_maximenu_variable;
  $maximenu = $_om_maximenu_variable;

  $out = array();  
  if (!empty($maximenu)) {
    foreach ($maximenu as $menu_key => $menu_content) {
      $visibility_values = isset($menu_content['menu_visibility']) ? $menu_content['menu_visibility']: array('visibility' => 0, 'pages' => '');
      $visibility = om_maximenu_menu_visibility($visibility_values);
      if (($menu_content['output'] != 'block') && $visibility) {  
        om_maximenu_asset_loader($menu_content);
        //$out[$menu_key]['title'] = $menu_content['title'];
        $out[$menu_key]['output'] = $menu_content['output'];
        $out[$menu_key]['content'] = om_maximenu_links_render($maximenu[$menu_key], $menu_key);     
      }
    }
  }
  return $out;  
}


/**
 * OM Maximenu links rendering engine
 *
 */
function om_maximenu_links_render($links = array(), $menu_key = 1) {
  // immediately returns nothing if no links
  if (!isset($links['links'])) return NULL;
      
  // menu name
  $maximenu_name = om_string_name($links['title']); 
  
  $out = '<ul id="om-menu-' . $maximenu_name . '" class="om-menu">';  

  // sort by weight
  uasort($links['links'], 'om_sort_by_weight'); 
  
  // first, last classes
  $count = 0;
  $total = count($links['links']);  
  
  foreach ($links['links'] as $key => $content) {

    // user roles and permissions    
    $permission = om_maximenu_link_visible($content['roles']);
    
    if (!empty($permission)) {    
      $count++;

      // php on title
      if (isset($content['php_option']) && ($content['php_option'] == 1)) {
        ob_start();
        $content['link_title'] = eval($content['link_title']);
        //$output = ob_get_contents();
        ob_end_clean(); 
      }
      
       // span id
      $span_id = (!empty($content['id'])) ? 'id="' . om_string_name($content['id']) . '"': '';        

      // span title
      $span_title = (!empty($content['description']) && ($content['description_option'] == 'hover')) ? 'title="' . check_plain($content['description']) . '"': ''; 
    
      // link content      
      $link_option = om_maximenu_link_option($content);
      
      // link attributes
      $attributes = om_maximenu_link_attrib($content);
                  
      $out .= '<li id="om-leaf-' . $links['code'] . '-' . $key . '" class="' . om_maximenu_link_classes($content, $permission, $count, $total) . '">';    

		  // link is disabled when it's on its page
      $uri = trim($_SERVER['REQUEST_URI']);
		  $path_relative = base_path() . $content['path'];

      // Drupal doesn't accept # as valid path on field input
      $options = array();       
      $options['query']    = isset($content['path_query']) ? om_path_query($content['path_query']): array();
      $options['fragment'] = isset($content['path_fragment']) ? $content['path_fragment']: '';                
				
      if (!empty($content['path']) && ($path_relative != $uri)) {
        $path = ($content['path'] == '<front>') ? '': $content['path'];
        // there are too many things inside this $link_options which a regular l() can't handle
        //$out .= l($content['link_title'], $content['path'], array('attributes' => $attributes));
        $out .= '<a ' . drupal_attributes($attributes) . ' href="' . url($path, $options) . '">' . $link_option . '</a>';
      }
      elseif (!empty($options['query']) || !empty($options['fragment'])) {
        // base path is not necessary when path is empty but anchor is not or query, 
        $query    = !empty($content['path_query']) ? '?' . $content['path_query']: '';
        $fragment = !empty($options['fragment']) ? '#' . $options['fragment']: '';
        $out .= '<a ' . drupal_attributes($attributes) . ' href="' .  $query . $fragment . '">' . $link_option . '</a>';
      }      
      else {
        // useful when you just want a button for getting the content to show and not actually linking to anything
        $out .= '<span ' . $span_id . ' class="' . $attributes['class'] . '" ' . $span_title . '>' . $link_option . '</span>';
      }
            
      // finally, getting the content
      if (!isset($links['tabbed']) || ($links['tabbed'] == '')) $out .= om_maximenu_submenu_render($content['content']); 

      $out .= '</li>';
    } // permission closing
  } // foreach closing
  $out .= '</ul>';  
  
  // tabbed content
  if (isset($links['tabbed']) && ($links['tabbed'] <> '')) $out .= om_maximenu_tabbed_render($links); 
       
  // access on editing
  if (user_access('administer OM Maximenu')) $out .= '<div class="om-maximenu-edit">' . l(t('Maxedit'), 'admin/structure/om-maximenu/' . $menu_key . '/edit', array('query' => drupal_get_destination())) . '</div>';    
  
  return om_maximenu_wrapper($links, $out);
}


/**
 * OM Maximenu submenu content rendering engine
 *
 */
function om_maximenu_submenu_render($content = array()) {
  global $_om_maximenu_block_classes;
    
  $block_classes = !empty($_om_maximenu_block_classes) ? $_om_maximenu_block_classes: array();  

  if (!empty($content)) {
    // enclose the content with divs for efficient image border theming
    $out = '<div class="om-maximenu-content om-maximenu-content-nofade closed">';
    $out .= '<div class="om-maximenu-top"><div class="om-maximenu-top-left"></div><div class="om-maximenu-top-right"></div></div>';
    $out .= '<div class="om-maximenu-middle"><div class="om-maximenu-middle-left"><div class="om-maximenu-middle-right">';      
    $count = 0;
    uasort($content, 'om_sort_by_weight');  
    $total = count($content);
    foreach ($content as $key => $prop) {
        $count++;
        
        $module              = $prop['module'];
        $delta               = $prop['delta'];        
        $title               = stripslashes(trim($prop['title'])); // allows markups
        $path                = ($prop['title_path'] == '<front>') ? '': $prop['title_path'];        
        $options             = array();
        $options['query']    = isset($prop['title_path_query']) ? om_path_query($prop['title_path_query']): '';
        $options['fragment'] = isset($prop['title_path_fragment']) ? $prop['title_path_fragment']: '';        
        $block_title         = (!empty($prop['title_path'])) ? '<a href="' . url($path, $options) .'" title="' . $title . '">' . $title . '</a>': $title;      
        $block               = module_invoke($module, 'block_view', $delta ); 

        $classes = ($count == 1) ? ' first': '';
        if ($count == $total) $classes .= ' last'; 
        $classes .=  isset($block_classes[$module][$delta]) ? ' ' . $block_classes[$module][$delta]: ''; 
        
        $out .= '<div class="block block-' . $module . ' block-' . $module . '-id-' . $delta . $classes . '">';            
        $out .= ($title != '<none>') ? '<h3 class="title">' . $block_title . '</h3>': ''; 
        // 2 different outputs, 1 rendered and 1 array - I wonder why?
        $content = isset($block['content']) ? is_array($block['content']) ? drupal_render($block['content']): $block['content']: '' . ''; 
        $out .= '<div class="content">' . $content . '</div>';
        if (user_access('administer blocks')) $out .= om_maximenu_block_edit($module, $delta, $title);
        $out .= '</div>';            
    }

    $out .= '<div class="om-clearfix"></div></div></div></div>';      
    $out .= '<div class="om-maximenu-bottom"><div class="om-maximenu-bottom-left"></div><div class="om-maximenu-bottom-right"></div></div>';    
    $out .= '<div class="om-maximenu-arrow"></div>';    
    $out .= '<div class="om-maximenu-open"><input type="checkbox" value="" />' . t('Stay') . '</div>';
    $out .= '</div>'; 
    return $out;
  } 
}


/**
 * OM Maximenu tabbed content rendering engine
 *
 */
function om_maximenu_tabbed_render($links = array()) {
  global $_om_maximenu_block_classes;
    
  $block_classes = !empty($_om_maximenu_block_classes) ? $_om_maximenu_block_classes: array();  

  // tabbed contents
  $out = '<div class="om-maximenu-tabbed-content"><div class="om-maximenu-tabbed-content-inner">';

  foreach ($links['links'] as $key => $content) {

    // user roles and permissions
    $permission = om_maximenu_link_visible($content['roles']);
    
    if (!empty($permission) && !empty($content['content'])) {  
      $count = 0;
      
      uasort($content['content'], 'om_sort_by_weight'); 
      
      $total = count($content['content']); 
      
      $out .= '<div id="om-tabbed-content-' . $links['code'] . '-' . $key . '" class="om-tabbed-content om-tabbed-content-hide"><div class="om-tabbed-content-inner">'; 
           
      foreach ($content['content'] as $key => $prop) {
        $count++;
        
        $module              = $prop['module'];
        $delta               = $prop['delta'];        
        $title               = stripslashes(trim($prop['title'])); // allows markups
        $path                = ($prop['title_path'] == '<front>') ? '': $prop['title_path'];        
        $options             = array();
        $options['query']    = isset($prop['title_path_query']) ? om_path_query($prop['title_path_query']): '';
        $options['fragment'] = isset($prop['title_path_fragment']) ? $prop['title_path_fragment']: '';        
        $block_title         = (!empty($prop['title_path'])) ? '<a href="' . url($path, $options) .'" title="' . $title . '">' . $title . '</a>': $title;      
        $block               = module_invoke($module, 'block_view', $delta ); 

        $classes = ($count == 1) ? ' first': '';
        if ($count == $total) $classes .= ' last'; 
        $classes .=  isset($block_classes[$module][$delta]) ? ' ' . $block_classes[$module][$delta]: ''; 
        
        $out .= '<div class="block block-' . $module . ' block-' . $module . '-id-' . $delta . $classes . '">';            
        $out .= ($title != '<none>') ? '<h3 class="title">' . $block_title . '</h3>': ''; 
        // 2 different outputs, 1 rendered and 1 array - I wonder why?
        $content = isset($block['content']) ? is_array($block['content']) ? drupal_render($block['content']): $block['content']: '' . ''; 
        $out .= '<div class="content">' . $content . '</div>';
        if (user_access('administer blocks')) $out .= om_maximenu_block_edit($module, $delta, $title);
        $out .= '</div>';            
      }
      $out .= '<div class="om-clearfix"></div>';
      $out .= '</div></div>';        
    } 
  }
  $out .= '<div class="om-clearfix"></div>';
  $out .= '</div></div>'; 
  return $out;  
}


/**
 * Sorting by weight
 *
 */
function om_sort_by_weight($a, $b) {
  return $a['weight'] - $b['weight'];
}

/**
 * Converts string either to a title or a class
 *
 */
function om_string_name($string = NULL, $class = TRUE) {
  // for the purpose of just showing the classes and title
  // any tags are stripped off
  $string = trim(strip_tags($string));
  // if there are slashes
  $string = stripslashes($string);  
  // ignores characters other than a-z, A-Z, 0-9
  $string_class = preg_replace('/[^a-zA-Z0-9-]+/', ' ', $string);  
  // allows all characters
  $string_title = $string; 
  // space and underscore are converted depending on use
  $pattern = '/[ _]+/';     
  $string = ($class == FALSE) ? ucwords(preg_replace($pattern, ' ', $string_title)): strtolower(preg_replace($pattern, '-', $string_class));
  // if result is none for non-class
  if (empty($string) && $class == FALSE) $string = 'Script Link';
  
  return $string;
}



/**
 * Link Options
 *
 */
function om_maximenu_link_option($content = array()) {

  // img hover replacement class
  $icon_option = (isset($content['icon_option']) && ($content['icon_option'] == 1)) ? 'class="om-hover"': '';

  // title, icon options
  $link_title        = stripslashes($content['link_title']);
  $link_title_option = $content['link_title_option'];
  $link_icon         = !empty($content['path_icon']) ? '<img ' . $icon_option . ' src="' . url($content['path_icon']) . '" alt="' . om_string_name($link_title, FALSE) . '" />': '';
      
  if ($link_title_option == 'title_icon') {
    $link_option = $link_icon . ' ' . $link_title;
  }
  elseif ($link_title_option == 'icon') {
    $link_option = $link_icon;
  }
  else {
    $link_option = $link_title;
  }
  
  // description as subtitle
  $link_subtitle = (!empty($content['description']) && ($content['description_option'] != 'hover')) ? '<span class="om-subtitle">' . check_plain($content['description']) . '</span>': ''; 
        
  return $link_option . $link_subtitle;
}
  
  
/**
 * Link attribute
 *
 */
function om_maximenu_link_attrib($content = array()) {
  // initialize link attributes and classes
  $attributes = array();
      
  // id link attributes
  if (!empty($content['id'])) $attributes['id'] = om_string_name($content['id']);

  // user added custom classes
  $custom_classes = '';
  $classes = explode(' ', $content['class']);
  foreach ($classes as $ckey => $class) {
    $custom_classes .= ' ' . om_string_name($class);
  }
  
  // class link attributes      
  $attributes['class'] = 'om-link' . $custom_classes . ' link-' . om_string_name($content['link_title']);  
  
  // active class   
  $uri = trim($_SERVER['REQUEST_URI']);
  $path_relative = base_path() . $content['path'];	
	    
  if (($_GET['q'] == $content['path']) || ($path_relative == $uri)) $attributes['class'] .= ' active'; 
  
  if ((empty($content['path']) || ($content['path'] == '<front>')) && drupal_is_front_page()) $attributes['class'] .= ' active';   
  
  // link description options
  if (!empty($content['description']) && ($content['description_option'] == 'hover')) $attributes['title'] = check_plain($content['description']); 

  // relationship
  if (!empty($content['rel'])) $attributes['rel'] = $content['rel'];

  // target
  if (!empty($content['target']) && ($content['target'] != '_self')) $attributes['target'] = $content['target'];

  return $attributes;
} 


/**
 * Classes aggregator
 *
 */
function om_maximenu_classes($links = array()) {
  $classes = array();
  
  // bubble, simple, no_style  
  $classes[] = $links['skin'];
  
  // block, float, main_menu  
  $classes[] = $links['output'];
  
  if ($links['output'] == 'block') {
    // row, column
    $classes[] = isset($links['block_options']['stacking']) ? $links['block_options']['stacking']: 'row';   
    // up, right, down, left
    $classes[] = isset($links['block_options']['direction']) ? $links['block_options']['direction']: 'block-down';   
  }
  elseif ($links['output'] == 'float') {
    // left, middle, right
    $classes[] = isset($links['float_options']['orientation']) ? $links['float_options']['orientation']: 'horizontal';       
    $classes[] = isset($links['float_options']['stacking']) ? $links['float_options']['stacking']: 'row';   
    $classes[] = isset($links['float_options']['x_origin']) ? $links['float_options']['x_origin']: 'left';   
    $classes[] = isset($links['float_options']['y_origin']) ? $links['float_options']['y_origin']: 'bottom';   
  }
  elseif ($links['output'] == 'main_menu') {
    // no class
  }  
  
  // fisheye dock class
  if (isset($links['float_options']['dock'])) $classes[] = 'dock';
  
  $out = '';
  foreach ($classes as $key => $class) {
    $out .= ' om-maximenu-' . om_string_name($class);   
  }
  return $out;
}   


/**
 * OM Maximenu inline styling
 *
 */
function om_maximenu_inline_style($links = array()) {
  $out = '';
  if ($links['output'] == 'block') {
    // do nothing now, maybe in the future
  }  
  elseif ($links['output'] == 'float') {
    // fixed/absolute
    $position    = (isset($links['float_options']['position']) && ($links['float_options']['position'] == 'absolute')) ? 'absolute': 'fixed';   
    // left/middle/right
    if (isset($links['float_options']['x_origin']) && ($links['float_options']['x_origin'] != 'middle')) {
      $x_origin    = isset($links['float_options']['x_origin']) ? $links['float_options']['x_origin']: 'left';     
      $x_value     = isset($links['float_options']['x_value']) ? $links['float_options']['x_value']: 10;
      $x_px        = ($x_value != 0) ? 'px': '';
      $x_prop      = $x_origin . ': ' . $x_value . $x_px . '; '; 
    } 
    else {
      $x_prop = ''; // jquery will determine the position
    }  
    // top/bottom
    $y_origin    = isset($links['float_options']['y_origin']) ? $links['float_options']['y_origin']: 'bottom';     
    $y_value     = isset($links['float_options']['y_value']) ? $links['float_options']['y_value']: 10; 
    $y_px        = ($y_value != 0) ? 'px': '';    
    $y_prop      = $y_origin . ': ' . $y_value . $y_px . '; ';
               
    $out = ' style="position: ' . $position . '; ' . $x_prop  . $y_prop . '" ';   
  }
  elseif ($links['output'] == 'main_menu') {
    // do nothing now, maybe in the future
  }
  
  return $out;
}


/**
 * Link classes processor
 *
 */
function om_maximenu_link_classes($content = array(), $permission = array(), $count = 0, $total = 0) {

  // initialize link classes
  $classes = array();     
                        
  // standard li classes
  if ($count == 1) $classes[] = 'first';
  if ($count == $total) $classes[] = 'last';  
      
  // link title class
  $classes[] = 'leaf-' . $content['link_title'];     

  // active class       
  if ($_GET['q'] == $content['path']) $classes[] = 'active'; 
      
  if ((empty($content['path']) || ($content['path'] == '<front>')) && drupal_is_front_page()) $classes[] = 'active';   
      
  // user role class
  if (is_array($permission)) {
    foreach ($permission as $pkey => $pval) {
      $classes[] = 'leaf-role-' . $pval; 
    }  
  }      
        
  // title, icon options
  $link_title_option = $content['link_title_option'];
  if ($link_title_option == 'title_icon') {
    $classes[] = 'om-leaf-title-icon';
  }
  elseif ($link_title_option == 'icon') {
    $classes[] = 'om-leaf-icon';
  }
  
  $out = 'om-leaf';
  foreach ($classes as $key => $class) {
    $out .= ' ' . om_string_name($class);   
  }
  return $out;
}


/**
 * Link visibility per user role
 *
 */
function om_maximenu_link_visible($roles = array()) {
  global $user;

  // user roles  
  $user_id = $user->uid;  
  $user_roles = $user->roles;  
  // user permission to view the link
  // roles
  // 1 - anonymous
  // 2 - authenticated

  $permission = array();
  $anyone = 0;
  foreach ($roles as $role => $value) {
     // check roles
     if (($value != 0) && isset($user_roles[$role])) $permission[$role] = $user_roles[$role];
     // for all roles are unchecked
     if ($value != 0) $anyone++;       
  }
  // the link will be visible if the user has permission 
  if (!empty($permission)) {
    return $permission;
  }
  elseif ($anyone == 0) { // the links is for everybody
    return 1;
  }
  elseif ($user_id == 1) { // the user is a super user
    return 1;
  }
  else {
    return 0;
  }
}      


/**
 * OM Maximenu get styles from /skin directory
 * - reads directory names
 */
function om_maximenu_style_get() {
  $files = array();
  $dir = OM_MAXIMENU_PATH . '/skin/';
  $folders = scandir($dir);
  $excluded_files = array('.', '..', '.cvs', '.svn', '.git');

  foreach ($folders as $key => $val) {
    if (!in_array($val, $excluded_files)) (is_dir($dir . $val)) ? $files[$val] = om_string_name($val, FALSE): '';     
  }
  return $files;
}


/**
 * Mouse actions
 *
 */
function om_maximenu_action($action = 'hover', $menu = '', $displace = 0) {

  $displace_menu = ($displace == 1) ? "$('#om-maximenu-" . $menu . " li.om-leaf .om-maximenu-content').css('position', 'relative').addClass('om-maximenu-displace');": '';
  
  $click_fast = "
    jQuery(document).ready(function($){"
      . $displace_menu . "     
      $('#om-maximenu-" . $menu . " li.om-leaf .om-maximenu-content').removeClass('om-maximenu-content-nofade');
      $('#om-maximenu-" . $menu . " li.om-leaf .om-link').click(function() {
				if($(this).parent().hasClass('open')) {
				  $(this).parent().removeClass('open');
				}
				else {
				  $('#om-maximenu-" . $menu . " li.om-leaf').removeClass('open');
				  $(this).parent().addClass('open');
				}
        $(this).parent().siblings().children('.om-maximenu-content').hide();
        $(this).siblings('div.om-maximenu-content').toggle().pause();
        return false;
      });
    });  
    ";

  $click_slow = "
    jQuery(document).ready(function($){"
      . $displace_menu . "     
      $('#om-maximenu-" . $menu . " li.om-leaf .om-maximenu-content').removeClass('om-maximenu-content-nofade');
      $('#om-maximenu-" . $menu . " li.om-leaf .om-link').click(function() {
				if($(this).parent().hasClass('open')) {
				  $(this).parent().removeClass('open');
				}
				else {
				  $('#om-maximenu-" . $menu . " li.om-leaf').removeClass('open');
				  $(this).parent().addClass('open');
				}	
        $(this).parent().siblings().children('.om-maximenu-content').slideUp('slow');
        $(this).siblings('div.om-maximenu-content').toggle('slow').pause();
        return false;
      });
    }); 
    ";
              
  $hover_fade = "
    jQuery(document).ready(function($){"
      . $displace_menu . "     
      $('#om-maximenu-" . $menu . " li.om-leaf .om-maximenu-content').removeClass('om-maximenu-content-nofade');
      //$('#om-maximenu-" . $menu . " li.om-leaf').hover(omFadeIn,omFadeOut); //native jquery
      $('#om-maximenu-" . $menu . " li.om-leaf').hoverIntent(
      function(){ $('.om-maximenu-content.closed', this).fadeIn();},
      function(){ $('.om-maximenu-content.closed', this).fadeOut();}); //jquery hoverintent
      //$('#om-maximenu-" . $menu . " li.om-leaf').hoverIntent({
        //over: omFadeIn,
        //timeout: 500,
        //out: omFadeOut
      //}); 
    });
    ";

  $hover = "jQuery(document).ready(function($){" . $displace_menu . " });";
  
  switch($action) {
    case 'click_fast': 
      drupal_add_js($click_fast, "inline");     
      break;
    case 'click_slow': 
      drupal_add_js($click_slow, "inline");   
      break;
    case 'hover_fade': 
      drupal_add_js(OM_MAXIMENU_PATH . '/contrib/jquery.hoverIntent.minified.js'); 
      drupal_add_js($hover_fade, "inline");   
      break;      
    default: 
      if ($displace == 1) drupal_add_js($hover, "inline");           
      break;   
  }  
}


/**
 * Dock jquery plugin
 *
 */
function om_maximenu_dock($menu = '') {
  $add_js = "jQuery(document).ready(function($){ $('#om-menu-" . $menu . " .om-link img').resizeOnApproach(); });";
  drupal_add_js(OM_MAXIMENU_PATH . '/contrib/jquery.resizeOnApproach.min.js'); 
  drupal_add_js($add_js, "inline");
}


/**
 * Animated background slider
 *
 */
function om_maximenu_bg_slider($menu = NULL) {
  $bg_slide_js = "jQuery(document).ready(function($){ $('ul#om-menu-" . $menu . "').lavaLamp({ speed: 700 }); })";
  drupal_add_js(OM_MAXIMENU_PATH . '/contrib/jquery.lavalamp.min.js');   
  drupal_add_js($bg_slide_js, "inline");  
}


/**
 * Animated Link slider
 *
 */
function om_maximenu_link_slider($menu = NULL) {
  $link_slide_js = "
    jQuery(document).ready(function($) {
      linkHeight = $('#om-menu-" . $menu . " .om-leaf').height();
  
      $('#om-menu-" . $menu . " .om-leaf').css('height', linkHeight + 'px');
      $('#om-menu-" . $menu . " .om-leaf .om-link').css('overflow', 'hidden').css('height', 0);      
	    $('#om-menu-" . $menu . " .om-leaf').prepend('<span class=\"om-link-slider\"></span>'); 

	    $('#om-menu-" . $menu . " .om-leaf').each(function() { 
		    var textLink = $(this).find('.om-link').text(); 
		    $(this).find('span.om-link-slider').show().text(textLink); 
	    }); 

	    $('#om-menu-" . $menu . " .om-leaf').hover(
	      function() { 
	        $(this).find('span.om-link-slider').stop().animate({ lineHeight: 0, height: 0 }, 250);
	        $(this).find('.om-link').stop().animate({ height: linkHeight }, 250); 
	      },
	      function() { 
	        $(this).find('span.om-link-slider').stop().animate({ lineHeight: linkHeight, height: linkHeight }, 250);
	        $(this).find('.om-link').stop().animate({ height: 0 }, 250);
	    });
    });
  ";
  drupal_add_js($link_slide_js, "inline");
}


/**
 * Animated link giggle
 *
 */
function om_maximenu_jiggle($menu = NULL) {  
  
  $link_giggle_js = "
    jQuery(document).ready(function($){ 
	    // creates the target paths
	    var list_elements = '#om-menu-" . $menu . "' + ' li.om-leaf';
	    var link_elements = list_elements + ' .om-link';
	    var pad_out = 25;
	    var pad_in = 15;
	    var time = 150;
	    var multiplier = .8;
	    
  	  // initiates the timer used for the sliding animation
  	  var timer = 0;
	
	    // creates the jiggle animation for all list elements 
	    $(list_elements).each(function(i){
  		  // margin left = - ([width of element] + [total vertical padding of element])
	  	  $(this).css('margin-left','-180px');
	  	  // updates timer
	  	  timer = (timer*multiplier + time);
	  	  $(this).animate({ marginLeft: '0' }, timer);
	  	  $(this).animate({ marginLeft: '15px' }, timer);
	  	  $(this).animate({ marginLeft: '0' }, timer);
	    });

  	  // creates the hover-jiggle effect for all link elements 		
  	  $(link_elements).each(function(i){
	  	  $(this).hover(
		      function(){
			      $(this).animate({ paddingLeft: pad_out }, 150);
		      },		
		      function(){
		      	$(this).animate({ paddingLeft: pad_in }, 150);
		    });
	    });
    });";
  drupal_add_js($link_giggle_js, "inline");  
}
    

/**
 * Animated floating menu
 *
 */
function om_maximenu_scroll($menu = NULL) {    

  $scroll_float_js = "
    jQuery(document).ready(function($){ 
      var topYloc = null;

      $(window).scroll(function () { 
	      var scrollTop = $(document).scrollTop();
	      scrollTop = parseInt(scrollTop);
	
	      var offset = topYloc+scrollTop+'px';  
	      $('#om-maximenu-" . $menu . "').animate({top:offset},{duration:500,queue:false});
      });

	    topYloc = parseInt($('#om-maximenu-" . $menu . "').css('top').substring(0,$('#om-maximenu-" . $menu . "').css('top').indexOf('px')));
    });";
  drupal_add_js($scroll_float_js, "inline");  
}


/**
 * Tabbed Content
 *
 */
function om_maximenu_tabbed($menu = NULL, $option = 'normal') {    

  $tabbed_js = "
    jQuery(document).ready(function($){
      $('#om-maximenu-" . $menu . " ul.om-menu li.om-leaf').removeClass('active');          
      $('#om-maximenu-" . $menu . " ul.om-menu li.om-leaf:first-child').addClass('active');          
      $('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content div:first-child').removeClass('om-tabbed-content-hide'); 
               
      $('#om-maximenu-" . $menu . " .om-leaf .om-link').click(function () {
        var leafId = $(this).parent().attr('id');
        var tabbedContentId = leafId.replace('leaf', 'tabbed-content');
        $('#om-maximenu-" . $menu . " ul.om-menu li.om-leaf').removeClass('active');          
        $(this).parent().addClass('active');          
                      
        $('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content .om-tabbed-content').addClass('om-tabbed-content-hide');              
        $('#om-maximenu-" . $menu . " #' + tabbedContentId).removeClass('om-tabbed-content-hide');      
      });
    });  
  ";

  $flow = '';  
  if ($option == 'scrollv') $flow = "$('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content-inner').css('height', totalHeight + 'px');"; 
  if ($option == 'scrollh') $flow = "$('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content-inner').css('width', totalWidth + 'px');"; 
    
  $tabbed_scroll_js = "
    jQuery(document).ready(function($){
      var contentNum = $('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content-inner').children().size();
      var height = $('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content').height();
      var width = $('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content').width();
      var totalHeight = height * contentNum;
      var totalWidth = width * contentNum;

      $('#om-maximenu-" . $menu . " .om-tabbed-content').css('height', height + 'px');
      $('#om-maximenu-" . $menu . " .om-tabbed-content').css('width', width + 'px');
      " . $flow . "
                        
      $('#om-maximenu-" . $menu . " ul.om-menu li.om-leaf').removeClass('active');          
      $('#om-maximenu-" . $menu . " ul.om-menu li.om-leaf:first-child').addClass('active');          
      $('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content .om-tabbed-content').removeClass('om-tabbed-content-hide'); 
               
      $('#om-maximenu-" . $menu . " .om-leaf .om-link').click(function () {
        var leafId = $(this).parent().attr('id');
        var tabbedContentId = leafId.replace('leaf', 'tabbed-content');
                        
        $('#om-maximenu-" . $menu . " ul.om-menu li.om-leaf').removeClass('active');          
        $(this).parent().addClass('active');          

        $('#om-maximenu-" . $menu . " .om-maximenu-tabbed-content').scrollTo($('#' + tabbedContentId), 500);
      });
    });  
  ";  
  
  if ($option == 'normal') { 
    drupal_add_js($tabbed_js, "inline"); 
  }
  else {
    drupal_add_js(OM_MAXIMENU_PATH . '/contrib/jquery.scrollTo.min.js'); 
    drupal_add_js($tabbed_scroll_js, "inline");   
  }  
}


/**
 * Admin - hide/show options
 *
 */
function om_maximenu_admin_js() {
  
  $add_js = "
    jQuery(document).ready(function($){ 
      $('form#om-maximenu-admin select.om-maximenu-output').change(function() {
        var parent_wrapper = $(this).parent().parent().parent().attr('id');  
        var output_value = $(this).val(); 
        if(output_value == 'block') {
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-block-options').show();
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-float-options').hide();          
        }
        if(output_value == 'float') {
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-float-options').show();
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-block-options').hide();
        }     
        if(output_value == 'main_menu') {
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-float-options').hide();
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-block-options').hide();
        } 
      });
      $('form#om-maximenu-admin select.om-maximenu-x-origin').change(function() {
        var option_wrapper = $(this).parent().parent().parent().attr('id');  
        var x_origin_value = $(this).val(); 
        //alert(option_wrapper);
        if(x_origin_value == 'middle') {
          $('form#om-maximenu-admin #' + option_wrapper + ' fieldset.om-maximenu-non-middle-options').hide();
        }
        else {
          $('form#om-maximenu-admin #' + option_wrapper + ' fieldset.om-maximenu-non-middle-options').show();
        }     
      });     
    }); ";
  
  drupal_add_js($add_js, "inline");
}


/**
 * OM Maximenu wrapper
 *
 */
function om_maximenu_wrapper($links = array(), $content = NULL) {
  $out = '';
  if (!empty($content)) {
    global $user;
    $maximenu_name = om_string_name($links['title']); 
    $code     = 'om-u'. $user->uid . '-' . mt_rand();
    $link_code = isset($links['code']) ? trim($links['code']): $code;  
    $out .= '<div id="om-maximenu-' . $maximenu_name . '" class="om-maximenu' . om_maximenu_classes($links) . ' code-' . $link_code . '"' . om_maximenu_inline_style($links) . '>';      
    //$out .= '<div id="om-maximenu-' . $maximenu_name . '-outer" class="om-maximenu-wrapper-outer">';
    $out .= ($links['output'] == 'float') ? '<div id="om-maximenu-' . $maximenu_name . '-inner" class="om-maximenu-wrapper-inner">' . $content . '</div>': $content;
    //$out .= '</div>';
    $out .= '</div>';    
  }
  return $out;
}


/**
 * Block edit link
 *
 */
function om_maximenu_block_edit($module, $delta, $title = NULL) {
  $vars = array();
  // Display 'edit block' for custom blocks.
  if ($module == 'block') {
    $options = array(
      'attributes' => array(
        'title' => t('Edit this block\'s content.'),
        'class' => array('block-edit'),
      ),
      'query' => drupal_get_destination(),
      'html' => TRUE,
    );     
    $vars[] = l( t('edit block'), 'admin/structure/block/manage/' . $module . '/' . $delta . '/configure', $options);
  }
  // Display 'configure' for other blocks.
  else {
    $options = array(
      'attributes' => array(
        'title' => t('Configure ' . $title),
        'class' =>  array('block-config'),
      ),
      'query' => drupal_get_destination(),
      'html' => TRUE,
    );    
    $vars[] = l( t('configure'), 'admin/structure/block/manage/' . $module . '/' . $delta . '/configure', $options);
  }
  // Display 'edit view' for Views blocks.
  if ($module == 'views' && user_access('administer views')) {
    list($view_name, $view_block) = explode('-block', $delta);
    $options = array(
      'attributes' => array(
        'title' => t('Edit this view.'),
        'class' =>  array('block-edit-view'),
      ),
      'query' => drupal_get_destination(),
      'html' => TRUE,
    );
    $vars[] = l( t('edit view'), 'admin/structure/views/view/' . $view_name . '/edit/block' . $view_block, $options);
  }
  // Display 'edit menu' for Menu blocks.
  elseif ((($module == 'menu') || ($module == 'system')) && user_access('administer menu')) {
    $menu_name = $delta;
    $options = array(
      'attributes' => array(
        'title' => t('Edit this menu.'),
        'class' =>  array('block-edit-menu'),
      ),
      'query' => drupal_get_destination(),
      'html' => TRUE,
    );
    $vars[] = l( t('edit menu'), 'admin/structure/menu/manage/' . $menu_name, $options);
  }

  $out = '<div class="edit-block">' . implode(' ', $vars) . '</div>';
  return $out;
}


/**
 * Menu Visibility
 *
 */
function om_maximenu_menu_visibility($menu_visibility = FALSE) {
  if ($menu_visibility['pages']) {
    if ($menu_visibility['visibility'] < 2) {
      $path = drupal_get_path_alias($_GET['q']);
      $page_match = drupal_match_path($path, $menu_visibility['pages']);
      if ($path != $_GET['q']) {
        $page_match = $page_match || drupal_match_path($_GET['q'], $menu_visibility['pages']);
      }
      $page_match = !($menu_visibility['visibility'] xor $page_match);
    }
    elseif (module_exists('php')) {
        $page_match = php_eval($menu_visibility['pages']);
    }
    else {
      $page_match = FALSE;
    }
  }
  else {
      $page_match = TRUE;
  }
  return $page_match;
}


/**
 * Block Configure Page - Get delta visiblity value form _om_maximenu_variable;
 *
 */
function om_maximenu_form_alter(&$form, $form_state, $form_id) { 
  switch($form_id) {
    case 'block_admin_configure':
      if($form['module']['#value'] == 'om_maximenu') $form['#submit'][] = '_om_maximenu_block_admin_configure_submit_handler';
      break;
    case 'block_admin_display_form':
      $form['#submit'][] = '_om_maximenu_block_admin_display_submit_handler';
      break;
  }
}


/**
 * Updates OM Maximenu variable on saving block configuration
 *
 */
function _om_maximenu_block_admin_configure_submit_handler(&$form, $form_state) {
  $delta = $form_state['values']['delta'];
  $delta = explode('-', $delta);
  
  if($delta[1] == 'maximenu') {
    global $_om_maximenu_variable;
    $maximenu = $_om_maximenu_variable;
    $delta = $delta[2];
    $maximenu[$delta]['menu_visibility']['visibility'] = $form_state['values']['visibility'];   
    $maximenu[$delta]['menu_visibility']['pages'] = $form_state['values']['pages'];
    variable_set('om_maximenu', $maximenu);
  }
}


/**
 * Updates OM Maximenu variable on saving block display
 *
 */
function _om_maximenu_block_admin_display_submit_handler(&$form, $form_state) {
  global $_om_maximenu_variable;
  $maximenu = $_om_maximenu_variable;
  $blocks = $form_state['values']['blocks'];
  
  $count = 0;
  foreach ($blocks as $module_delta => $values) {
    if ($values['module'] == 'om_maximenu') {
      $delta = explode('-', $values['delta']);
      $delta = $delta[2];
      $maximenu[$delta]['block_options']['region'] = $values['region'];     
      $maximenu[$delta]['block_options']['weight'] = $values['weight']; 
      $count++;           
    }
  }
  // will only process if OM Maximenu blocks are present
  if ($count != 0) variable_set('om_maximenu', $maximenu);  
}


/**
 * Block db updater
 *
 */
function om_maximenu_update_blocks_db($menu_id = 0, $maximenu_content = array()) {
  if ($menu_id != 0) {
    if ($maximenu_content['output'] == 'block') {
    
      $theme_default = variable_get('theme_default', 'garland');

      $existing = db_select('block', 'n')
        ->fields('n')
        ->condition('module', 'om_maximenu', '=')
        ->condition('delta', 'om-maximenu-' . $menu_id, '=') 
        ->condition('theme', $theme_default, '=')                           
        ->execute()
        ->fetchObject(); 

      if (!empty($existing->delta)) {                      
        // if the menu is existing   
        $status = (!empty($maximenu_content['block_options']['region'])) ? 1: 0;
        
        db_update('block')
          ->fields(array(
            'status'     => $status,
            'weight'     => $maximenu_content['block_options']['weight'],                                  
            'region'     => $maximenu_content['block_options']['region'],  
            'visibility' => (int) $maximenu_content['menu_visibility']['visibility'],
            'pages'      => trim($maximenu_content['menu_visibility']['pages']),
          ))
          ->condition('module', 'om_maximenu')
          ->condition('delta', 'om-maximenu-' . $menu_id) 
          ->condition('theme', $theme_default)                             
          ->execute();
      }
      else {  
        // because we are handling the visibility, etc., we have to handle everything to synch with blocks settings
        $status = (!empty($maximenu_content['block_options']['region'])) ? 1: 0;

        db_insert('block')
          ->fields(array(
            'module'     => 'om_maximenu', 
            'delta'      => 'om-maximenu-' . $menu_id,                                     
            'theme'      => $theme_default,   
            'status'     => $status,                   
            'weight'     => $maximenu_content['block_options']['weight'],                                 
            'region'     => $maximenu_content['block_options']['region'],  
            'visibility' => (int) $maximenu_content['menu_visibility']['visibility'],
            'pages'      => trim($maximenu_content['menu_visibility']['pages']),
            'cache'      => -1,  // no cache
          ))->execute();
      }                     
    }
    elseif ($maximenu_content['output'] == 'float') {      
      // menu as float must be deleted from block table
      db_delete('block')->condition('module', 'om_maximenu')->condition('delta', 'om-maximenu-' . $menu_id)->execute();      
    }
    elseif ($maximenu_content['output'] == 'main_menu') {
      // menu as main_menu must be deleted from blocks table
      db_delete('block')->condition('module', 'om_maximenu')->condition('delta', 'om-maximenu-' . $menu_id)->execute();      
    }  
  }
}

          
/**
 * Get the hierarchy of menus
 *
 */
function om_menu_tree_options(&$options, $menus, $menu_name = 'navigation') {
  global $_om_maximenu_existing_menus;
  $existing = $_om_maximenu_existing_menus[$menu_name];
        
  static $level = 1; // count level for indents on divs on html file
  $tab = '-';        // init tab for indents on divs on html file
  // for source formating
  // calculates number of tabs
  for ($i = 1; $i < $level; $i++) {
    $tab .= '-';
  }    
  foreach ($menus as $mlid => $link_title) {
    if (isset($existing[$mlid])) {
      $level++; 
      $options[$mlid] = $tab . $link_title;
      om_menu_tree_options($options, $existing[$mlid], $menu_name);
      $level--;       
    }
    else {
      $options[$mlid] = $tab . $link_title;
    }
  }
}


/**
 * Path queries 
 *
 * On D7 $option['query'] becomes purely an array
 * cannot accept anymore a string
 *
 */
function om_path_query($path_query = array()) {        
  $out = array();
  $queries = explode('&', $path_query);

  if (!empty($queries)) {
    foreach ($queries as $kquery => $query) {
      if (!empty($query)) {
        $query = explode('=', $query);
        if (!empty($query)) $out[$query[0]] = $query[1];
      }
    }
  }
  return $out;
} 
